@@ -129,12 +129,14 @@ static janus_mutex writable_mutex;
 
 static int janus_wsevh_callback(struct lws *wsi, enum lws_callback_reasons reason, void *user, void *in, size_t len);
 static struct lws_protocols protocols[] = {
-	{ NULL, janus_wsevh_callback, sizeof(janus_wsevh_client), 0 },	/* Subprotocol will be configurable */
+	{ "janus-event-handlers", janus_wsevh_callback, sizeof(janus_wsevh_client), 0 },	/* Subprotocol will be configurable */
 	{ NULL, NULL, 0, 0 }
 };
 static const struct lws_extension exts[] = {
+#ifndef LWS_WITHOUT_EXTENSIONS
 	{ "permessage-deflate", lws_extension_callback_pm_deflate, "permessage-deflate; client_max_window_bits" },
 	{ "deflate-frame", lws_extension_callback_pm_deflate, "deflate_frame" },
+#endif
 	{ NULL, NULL, NULL }
 };
 
@@ -264,7 +266,7 @@ int janus_wsevh_init(const char *config_path) {
 		goto error;
 	}
 	if(strcasecmp(protocol, "ws") || !strlen(address)) {
-		JANUS_LOG(LOG_FATAL, "Invalid address (only ws:// and wss:// addresses are supported)\n");
+		JANUS_LOG(LOG_FATAL, "Invalid address (only ws:// addresses are supported)\n");
 		JANUS_LOG(LOG_FATAL, "  -- Protocol: %s\n", protocol);
 		JANUS_LOG(LOG_FATAL, "  -- Address:  %s\n", address);
 		JANUS_LOG(LOG_FATAL, "  -- Path:     %s\n", path);
@@ -658,7 +660,7 @@ static int janus_wsevh_callback(struct lws *wsi, enum lws_callback_reasons reaso
 				char *event = g_async_queue_try_pop(messages);
 				if(event && !g_atomic_int_get(&stopping)) {
 					/* Gotcha! */
-					int buflen = LWS_SEND_BUFFER_PRE_PADDING + strlen(event) + LWS_SEND_BUFFER_POST_PADDING;
+					int buflen = LWS_PRE + strlen(event);
 					if(ws_client->buffer == NULL) {
 						/* Let's allocate a shared buffer */
 						JANUS_LOG(LOG_VERB, "Allocating %d bytes (event is %zu bytes)\n", buflen, strlen(event));
@@ -671,14 +673,14 @@ static int janus_wsevh_callback(struct lws *wsi, enum lws_callback_reasons reaso
 						ws_client->buflen = buflen;
 						ws_client->buffer = g_realloc(ws_client->buffer, buflen);
 					}
-					memcpy(ws_client->buffer + LWS_SEND_BUFFER_PRE_PADDING, event, strlen(event));
+					memcpy(ws_client->buffer + LWS_PRE, event, strlen(event));
 					JANUS_LOG(LOG_VERB, "Sending WebSocket message (%zu bytes)...\n", strlen(event));
-					int sent = lws_write(wsi, ws_client->buffer + LWS_SEND_BUFFER_PRE_PADDING, strlen(event), LWS_WRITE_TEXT);
+					int sent = lws_write(wsi, ws_client->buffer + LWS_PRE, strlen(event), LWS_WRITE_TEXT);
 					JANUS_LOG(LOG_VERB, "  -- Sent %d/%zu bytes\n", sent, strlen(event));
 					if(sent > -1 && sent < (int)strlen(event)) {
 						/* We couldn't send everything in a single write, we'll complete this in the next round */
 						ws_client->bufpending = strlen(event) - sent;
-						ws_client->bufoffset = LWS_SEND_BUFFER_PRE_PADDING + sent;
+						ws_client->bufoffset = LWS_PRE + sent;
 						JANUS_LOG(LOG_VERB, "  -- Couldn't write all bytes (%d missing), setting offset %d\n",
 							ws_client->bufpending, ws_client->bufoffset);
 					}
