@@ -76,7 +76,8 @@ $(document).ready(function() {
 				return;
 			}
 			// Make sure Insertable Streams are supported too
-			if(!RTCRtpSender.prototype.createEncodedAudioStreams ||
+			if(!RTCRtpSender.prototype.createEncodedStreams &&
+					!RTCRtpSender.prototype.createEncodedAudioStreams &&
 					!RTCRtpSender.prototype.createEncodedVideoStreams) {
 				bootbox.alert("Insertable Streams not supported by this browser... ");
 				return;
@@ -200,7 +201,7 @@ $(document).ready(function() {
 									Janus.debug(" ::: Got a local stream :::", stream);
 									if($('#myvideo').length === 0) {
 										$('#videos').removeClass('hide').show();
-										$('#videoleft').append('<video class="rounded centered" id="myvideo" width=320 height=240 autoplay playsinline muted="muted"/>');
+										$('#videoleft').append('<video class="rounded centered" id="myvideo" width="100%" height="100%" autoplay playsinline muted="muted"/>');
 									}
 									Janus.attachMediaStream($('#myvideo').get(0), stream);
 									$("#myvideo").get(0).muted = "muted";
@@ -215,7 +216,7 @@ $(document).ready(function() {
 											}
 										});
 										// No remote video yet
-										$('#videoright').append('<video class="rounded centered" id="waitingvideo" width=320 height=240 />');
+										$('#videoright').append('<video class="rounded centered" id="waitingvideo" width="100%" height="100%" />');
 										if(spinner == null) {
 											var target = document.getElementById('videoright');
 											spinner = new Spinner({top:100}).spin(target);
@@ -245,7 +246,7 @@ $(document).ready(function() {
 									if($('#peervideo').length === 0) {
 										addButtons = true;
 										$('#videos').removeClass('hide').show();
-										$('#videoright').append('<video class="rounded centered hide" id="peervideo" width=320 height=240 autoplay playsinline/>');
+										$('#videoright').append('<video class="rounded centered hide" id="peervideo" width="100%" height="100%" autoplay playsinline/>');
 										// Show the video, hide the spinner and show the resolution when we get a playing event
 										$("#peervideo").bind("playing", function () {
 											$('#waitingvideo').remove();
