@@ -67,7 +67,9 @@ var vcodec = (getQueryStringValue("vcodec") !== "" ? getQueryStringValue("vcodec
 var vprofile = (getQueryStringValue("vprofile") !== "" ? getQueryStringValue("vprofile") : null);
 var doSimulcast = (getQueryStringValue("simulcast") === "yes" || getQueryStringValue("simulcast") === "true");
 var doSimulcast2 = (getQueryStringValue("simulcast2") === "yes" || getQueryStringValue("simulcast2") === "true");
-
+var recordData = (getQueryStringValue("data") !== "" ? getQueryStringValue("data") : null);
+if(recordData !== "text" && recordData !== "binary")
+	recordData = null;
 
 $(document).ready(function() {
 	// Initialize the library (all console debuggers enabled)
@@ -147,7 +149,7 @@ $(document).ready(function() {
 												recordplay.createAnswer(
 													{
 														jsep: jsep,
-														media: { audioSend: false, videoSend: false },	// We want recvonly audio/video
+														media: { audioSend: false, videoSend: false, data: true },	// We want recvonly audio/video
 														success: function(jsep) {
 															Janus.debug("Got SDP!", jsep);
 															var body = { request: "start" };
@@ -241,7 +243,7 @@ $(document).ready(function() {
 									$('#stop').unbind('click').click(stop);
 									$('#video').removeClass('hide').show();
 									if($('#thevideo').length === 0)
-										$('#videobox').append('<video class="rounded centered" id="thevideo" width=320 height=240 autoplay playsinline muted="muted"/>');
+										$('#videobox').append('<video class="rounded centered" id="thevideo" width="100%" height="100%" autoplay playsinline muted="muted"/>');
 									Janus.attachMediaStream($('#thevideo').get(0), stream);
 									$("#thevideo").get(0).muted = "muted";
 									if(recordplay.webrtcStuff.pc.iceConnectionState !== "completed" &&
@@ -279,9 +281,9 @@ $(document).ready(function() {
 										$('#videotitle').html(selectedRecordingInfo);
 										$('#stop').unbind('click').click(stop);
 										$('#video').removeClass('hide').show();
-										$('#videobox').append('<video class="rounded centered hide" id="thevideo" width=320 height=240 autoplay playsinline/>');
+										$('#videobox').append('<video class="rounded centered hide" id="thevideo" width="100%" height="100%" autoplay playsinline/>');
 										// No remote video yet
-										$('#videobox').append('<video class="rounded centered" id="waitingvideo" width=320 height=240 />');
+										$('#videobox').append('<video class="rounded centered" id="waitingvideo" width="100%" height="100%" />');
 										if(spinner == null) {
 											var target = document.getElementById('videobox');
 											spinner = new Spinner({top:100}).spin(target);
@@ -314,6 +316,20 @@ $(document).ready(function() {
 										$('#thevideo').removeClass('hide').show();
 									}
 								},
+								ondataopen: function(data) {
+									Janus.log("The DataChannel is available!");
+									$('#datafield').parent().removeClass('hide');
+									if(playing === false) {
+										// We're recording, use this field to send data
+										$('#datafield').attr('placeholder', 'Write a message to record');
+										$('#datafield').removeAttr('disabled');
+									}
+								},
+								ondata: function(data) {
+									Janus.debug("We got data from the DataChannel!", data);
+									if(playing === true)
+										$('#datafield').val(data);
+								},
 								oncleanup: function() {
 									Janus.log(" ::: Got a cleanup notification :::");
 									// FIXME Reset status
@@ -324,6 +340,8 @@ $(document).ready(function() {
 									$('#videobox').empty();
 									$("#videobox").parent().unblock();
 									$('#video').hide();
+									$('#datafield').attr('disabled', true).attr('placeholder', '').val('');
+									$('#datafield').parent().addClass('hide');
 									recording = false;
 									playing = false;
 									$('#record').removeAttr('disabled').click(startRecording);
@@ -349,17 +367,29 @@ $(document).ready(function() {
 	}});
 });
 
-function checkEnter(field, event) {
+function checkEnter(event) {
 	var theCode = event.keyCode ? event.keyCode : event.which ? event.which : event.charCode;
 	if(theCode == 13) {
-		if(field.id == 'name')
-			insertName();
+		sendData();
 		return false;
 	} else {
 		return true;
 	}
 }
 
+function sendData() {
+	var data = $('#datafield').val();
+	if(data === "") {
+		bootbox.alert('Insert a message to send on the DataChannel');
+		return;
+	}
+	recordplay.data({
+		text: data,
+		error: function(reason) { bootbox.alert(reason); },
+		success: function() { $('#datafield').val(''); },
+	});
+}
+
 function updateRecsList() {
 	$('#list').unbind('click');
 	$('#update-list').addClass('fa-spin');
@@ -426,7 +456,9 @@ function startRecording() {
 
 		recordplay.createOffer(
 			{
-				// By default, it's sendrecv for audio and video... no datachannels
+				// By default, it's sendrecv for audio and video... no datachannels,
+				// unless we've passed the query string argument to record those too
+				media: { data: (recordData != null) },
 				// If you want to test simulcasting (Chrome and Firefox only), then
 				// pass a ?simulcast=true when opening this demo page: it will turn
 				// the following 'simulcast' property to pass to janus.js to true
@@ -444,6 +476,9 @@ function startRecording() {
 					// profile as well (e.g., ?vprofile=2 for VP9, or ?vprofile=42e01f for H.264)
 					if(vprofile)
 						body["videoprofile"] = vprofile;
+					// If we're going to send binary data, let's tell the plugin
+					if(recordData === "binary")
+						body["textdata"] = false;
 					recordplay.send({ message: body, jsep: jsep });
 				},
 				error: function(error) {
