@@ -193,6 +193,26 @@ function dataReady(id) {
 	// to throttle outgoing data and not send too much at a time.
 }
 
+function substreamChanged(id, substream) {
+	// If simulcast is used, this callback is invoked when the substream
+	// we're sending to this session changes: 0=low, 1=medium, 2=high
+	console.log("Substream changed for session " + id + ": " + substream);
+	// Let's send an event so that the user is aware
+	var event = { echotest: "event", videocodec: "vp8", substream: substream };
+	var jsonevent = JSON.stringify(event);
+	pushEvent(id, null, jsonevent, null);
+}
+
+function temporalLayerChanged(id, temporal) {
+	// If simulcast is used, this callback is invoked when the temporal
+	// layer we're sending to this session changes: 0=lowfps, 1=maxfps
+	console.log("Temporal layer changed for session " + id + ": " + temporal);
+	// Let's send an event so that the user is aware
+	var event = { echotest: "event", videocodec: "vp8", temporal: temporal };
+	var jsonevent = JSON.stringify(event);
+	pushEvent(id, null, jsonevent, null);
+}
+
 function resumeScheduler() {
 	// This is the function responsible for resuming coroutines associated
 	// with whatever is relevant to the JS script, e.g., for this script,
@@ -240,6 +260,17 @@ function processRequest(id, msg) {
 	if(msg["bitrate"] !== null && msg["bitrate"] !== undefined) {
 		setBitrate(id, msg["bitrate"]);
 	}
+	if(msg["substream"] !== null && msg["substream"] !== undefined) {
+		setSubstream(id, msg["substream"]);
+		sendPli(id);
+	}
+	if(msg["temporal"] !== null && msg["temporal"] !== undefined) {
+		setTemporalLayer(id, msg["temporal"]);
+		sendPli(id);
+	}
+	if(msg["keyframe"] !== null && msg["keyframe"] !== undefined) {
+		sendPli(id);
+	}
 	if(msg["record"] === true) {
 		var fnbase = msg["filename"];
 		if(!fnbase) {
