@@ -65,6 +65,7 @@ janus_plugin *create(void) {
  * - \c incoming_rtp(): a callback to notify you a peer has sent you a RTP packet;
  * - \c incoming_rtcp(): a callback to notify you a peer has sent you a RTCP message;
  * - \c incoming_data(): a callback to notify you a peer has sent you a message on a SCTP DataChannel;
+ * - \c data_ready(): a callback to notify you data can be sent on the SCTP DataChannel;
  * - \c slow_link(): a callback to notify you a peer has sent a lot of NACKs recently, and the media path may be slow;
  * - \c hangup_media(): a callback to notify you the peer PeerConnection has been closed (e.g., after a DTLS alert);
  * - \c query_session(): this method is called by the core to get plugin-specific info on a session between you and a peer;
@@ -170,7 +171,7 @@ janus_plugin *create(void) {
  * Janus instance or it will crash.
  *
  */
-#define JANUS_PLUGIN_API_VERSION	14
+#define JANUS_PLUGIN_API_VERSION	15
 
 /*! \brief Initialization of all plugin properties to NULL
  *
@@ -205,6 +206,7 @@ static janus_plugin janus_echotest_plugin =
 		.incoming_rtp = NULL,			\
 		.incoming_rtcp = NULL,			\
 		.incoming_data = NULL,			\
+		.data_ready = NULL,				\
 		.slow_link = NULL,				\
 		.hangup_media = NULL,			\
 		.destroy_session = NULL,		\
@@ -310,6 +312,11 @@ struct janus_plugin {
 	 * @param[in] handle The plugin/gateway session used for this peer
 	 * @param[in] packet The message data and related info */
 	void (* const incoming_data)(janus_plugin_session *handle, janus_plugin_data *packet);
+	/*! \brief Method to be notified about the fact that the datachannel is ready to be written
+	 * \note This is not only called when the PeerConnection first becomes available, but also
+	 * when the SCTP socket becomes writable again, e.g., because the internal buffer is empty.
+	 * @param[in] handle The plugin/gateway session used for this peer */
+	void (* const data_ready)(janus_plugin_session *handle);
 	/*! \brief Method to be notified by the core when too many NACKs have
 	 * been received or sent by Janus, and so a slow or potentially
 	 * unreliable network is to be expected for this peer
@@ -365,6 +372,9 @@ struct janus_callbacks {
 	 * @param[in] packet The RTCP packet and related data */
 	void (* const relay_rtcp)(janus_plugin_session *handle, janus_plugin_rtcp *packet);
 	/*! \brief Callback to relay SCTP/DataChannel messages to a peer
+	 * @note The protocol is only used for the first message sent on a new data
+	 * channel, as it will be used to create it; it will be ignored for following
+	 * messages on the same label, so you can set NULL after that
 	 * @param[in] handle The plugin/gateway session that will be used for this peer
 	 * @param[in] packet The message data and related info */
 	void (* const relay_data)(janus_plugin_session *handle, janus_plugin_data *packet);
@@ -596,6 +606,8 @@ void janus_plugin_rtcp_reset(janus_plugin_rtcp *packet);
 struct janus_plugin_data {
 	/*! \brief The label this message belongs to */
 	char *label;
+	/*! \brief The subprotocol this message refers to */
+	char *protocol;
 	/*! \brief Whether the message data is text (default=FALSE) or binary */
 	gboolean binary;
 	/*! \brief The message data */
