@@ -430,16 +430,15 @@ janus_sdp *janus_sdp_parse(const char *sdp, char *error, size_t errlen) {
 						m->proto = g_strdup(proto);
 						m->direction = JANUS_SDP_SENDRECV;
 						m->c_ipv4 = TRUE;
-						if(m->port > 0 || m->type == JANUS_SDP_APPLICATION) {
-							/* Now let's check the payload types/formats */
-							gchar **mline_parts = g_strsplit(line+2, " ", -1);
-							if(!mline_parts) {
-								janus_sdp_mline_destroy(m);
-								if(error)
-									g_snprintf(error, errlen, "Invalid m= line (no payload types/formats): %s", line);
-								success = FALSE;
-								break;
-							}
+						/* Now let's check the payload types/formats */
+						gchar **mline_parts = g_strsplit(line+2, " ", -1);
+						if(!mline_parts && (m->port > 0 || m->type == JANUS_SDP_APPLICATION)) {
+							janus_sdp_mline_destroy(m);
+							if(error)
+								g_snprintf(error, errlen, "Invalid m= line (no payload types/formats): %s", line);
+							success = FALSE;
+							break;
+						} else {
 							int mindex = 0;
 							while(mline_parts[mindex]) {
 								if(mindex < 3) {
@@ -1218,7 +1217,7 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	}
 	if(audio_codec == NULL)
 		audio_codec = "opus";
-	const char *audio_rtpmap = janus_sdp_get_codec_rtpmap(audio_codec);
+	const char *audio_rtpmap = do_audio ? janus_sdp_get_codec_rtpmap(audio_codec) : NULL;
 	if(do_audio && audio_rtpmap == NULL) {
 		JANUS_LOG(LOG_ERR, "Unsupported audio codec '%s', can't prepare an offer\n", audio_codec);
 		va_end(args);
@@ -1234,7 +1233,7 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	}
 	if(video_codec == NULL)
 		video_codec = "vp8";
-	const char *video_rtpmap = janus_sdp_get_codec_rtpmap(video_codec);
+	const char *video_rtpmap = do_video ? janus_sdp_get_codec_rtpmap(video_codec) : NULL;
 	if(do_video && video_rtpmap == NULL) {
 		JANUS_LOG(LOG_ERR, "Unsupported video codec '%s', can't prepare an offer\n", video_codec);
 		va_end(args);
