@@ -47,17 +47,20 @@ janus_sdp *janus_sdp_preparse(void *ice_handle, const char *jsep_sdp, char *erro
 	GList *temp = parsed_sdp->m_lines;
 	while(temp) {
 		janus_sdp_mline *m = (janus_sdp_mline *)temp->data;
-		if(m->type == JANUS_SDP_AUDIO && m->port > 0) {
+		if(m->type == JANUS_SDP_AUDIO) {
 			*audio = *audio + 1;
-		} else if(m->type == JANUS_SDP_VIDEO && m->port > 0) {
+		} else if(m->type == JANUS_SDP_VIDEO) {
 			*video = *video + 1;
 		}
-		/* Preparse the mid as well */
+		/* Preparse the mid as well, and check if bundle-only is used */
 		GList *tempA = m->attributes;
 		while(tempA) {
 			janus_sdp_attribute *a = (janus_sdp_attribute *)tempA->data;
 			if(a->name) {
-				if(!strcasecmp(a->name, "mid")) {
+				if(!strcasecmp(a->name, "bundle-only") && m->port == 0) {
+					/* Port 0 but bundle-only is used, don't disable this m-line */
+					m->port = 9;
+				} else if(!strcasecmp(a->name, "mid")) {
 					/* Found mid attribute */
 					if(a->value == NULL) {
 						JANUS_LOG(LOG_ERR, "[%"SCNu64"] Invalid mid attribute (no value)\n", handle->handle_id);
@@ -89,6 +92,14 @@ janus_sdp *janus_sdp_preparse(void *ice_handle, const char *jsep_sdp, char *erro
 					}
 				}
 			}
+			/* If the m-line is disabled don't actually increase the count */
+			if(m->port == 0) {
+				if(m->type == JANUS_SDP_AUDIO) {
+					*audio = *audio - 1;
+				} else if(m->type == JANUS_SDP_VIDEO) {
+					*video = *video - 1;
+				}
+			}
 			tempA = tempA->next;
 		}
 		temp = temp->next;
@@ -104,7 +115,7 @@ janus_sdp *janus_sdp_preparse(void *ice_handle, const char *jsep_sdp, char *erro
 }
 
 /* Parse SDP */
-int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update) {
+int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean rids_hml, gboolean update) {
 	if(!ice_handle || !remote_sdp)
 		return -1;
 	janus_ice_handle *handle = (janus_ice_handle *)ice_handle;
@@ -418,6 +429,7 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 		}
 		/* Is simulcasting enabled, using rid? (we need to check this before parsing SSRCs) */
 		tempA = m->attributes;
+		stream->rids_hml = rids_hml;
 		while(tempA) {
 			janus_sdp_attribute *a = (janus_sdp_attribute *)tempA->data;
 			if(a->name && !strcasecmp(a->name, "rid") && a->value) {
@@ -427,12 +439,12 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 					JANUS_LOG(LOG_ERR, "[%"SCNu64"] Failed to parse rid attribute...\n", handle->handle_id);
 				} else {
 					JANUS_LOG(LOG_VERB, "[%"SCNu64"] Parsed rid: %s\n", handle->handle_id, rid);
-					if(stream->rid[0] == NULL) {
-						stream->rid[0] = g_strdup(rid);
+					if(stream->rid[rids_hml ? 2 : 0] == NULL) {
+						stream->rid[rids_hml ? 2 : 0] = g_strdup(rid);
 					} else if(stream->rid[1] == NULL) {
 						stream->rid[1] = g_strdup(rid);
-					} else if(stream->rid[2] == NULL) {
-						stream->rid[2] = g_strdup(rid);
+					} else if(stream->rid[rids_hml ? 0 : 2] == NULL) {
+						stream->rid[rids_hml ? 0 : 2] = g_strdup(rid);
 					} else {
 						JANUS_LOG(LOG_WARN, "[%"SCNu64"] Too many RTP Stream IDs, ignoring '%s'...\n", handle->handle_id, rid);
 					}
@@ -443,6 +455,16 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp, gboolean update)
 			}
 			tempA = tempA->next;
 		}
+		/* If rid is involved, check how many of them we have (it may be less than 3) */
+		if(stream->rid[0] == NULL && stream->rid[2] != NULL) {
+			stream->rid[0] = stream->rid[1];
+			stream->rid[1] = stream->rid[2];
+			stream->rid[2] = NULL;
+		}
+		if(stream->rid[0] == NULL && stream->rid[1] != NULL) {
+			stream->rid[0] = stream->rid[1];
+			stream->rid[1] = NULL;
+		}
 		/* Let's start figuring out the SSRCs, and any grouping that may be there */
 		stream->audio_ssrc_peer_new = 0;
 		stream->video_ssrc_peer_new[0] = 0;
@@ -738,6 +760,7 @@ int janus_sdp_parse_candidate(void *ice_stream, const char *candidate, int trick
 			GResolver *resolver = g_resolver_get_default();
 			g_resolver_lookup_by_name_async(resolver, rip, NULL,
 				(GAsyncReadyCallback)janus_sdp_mdns_resolved, mc);
+			g_object_unref(resolver);
 			return 0;
 		}
 		/* Add remote candidate */
@@ -926,8 +949,8 @@ int janus_sdp_parse_ssrc_group(void *ice_stream, const char *group_attr, int vid
 	janus_ice_handle *handle = stream->handle;
 	if(handle == NULL)
 		return -2;
-	if(!video)
-		return -3;
+	if(!video)	/* We only do rtx for video, return */
+		return 0;
 	if(stream->rid[0] != NULL) {
 		/* Simulcasting is rid-based, don't parse SSRCs for now */
 		return 0;
@@ -1233,14 +1256,14 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 		janus_sdp_mline *m = (janus_sdp_mline *)temp->data;
 		if(m->type == JANUS_SDP_AUDIO) {
 			audio++;
-			if(audio == 1) {
+			if(audio == 1 && m->port > 0) {
 				g_snprintf(buffer_part, sizeof(buffer_part),
 					" %s", handle->audio_mid ? handle->audio_mid : "audio");
 				g_strlcat(buffer, buffer_part, sizeof(buffer));
 			}
 		} else if(m->type == JANUS_SDP_VIDEO) {
 			video++;
-			if(video == 1) {
+			if(video == 1 && m->port > 0) {
 				g_snprintf(buffer_part, sizeof(buffer_part),
 					" %s", handle->video_mid ? handle->video_mid : "video");
 				g_strlcat(buffer, buffer_part, sizeof(buffer));
@@ -1249,7 +1272,7 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 		} else if(m->type == JANUS_SDP_APPLICATION) {
 			if(m->proto && (!strcasecmp(m->proto, "DTLS/SCTP") || !strcasecmp(m->proto, "UDP/DTLS/SCTP")))
 				data++;
-			if(data == 1) {
+			if(data == 1 && m->port > 0) {
 				g_snprintf(buffer_part, sizeof(buffer_part),
 					" %s", handle->data_mid ? handle->data_mid : "data");
 				g_strlcat(buffer, buffer_part, sizeof(buffer));
@@ -1494,23 +1517,24 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon, gboolean offer) {
 		if(m->type == JANUS_SDP_VIDEO && stream->rid[0] != NULL) {
 			char rids[50];
 			rids[0] = '\0';
-			int i=0;
-			for(i=0; i<3; i++) {
-				if(stream->rid[i] == NULL)
+			int i=0, index=0;
+			for(i=2; i>=0; i--) {
+				index = (stream->rids_hml ? i : (2-i));
+				if(stream->rid[index] == NULL)
 					continue;
-				a = janus_sdp_attribute_create("rid", "%s recv", stream->rid[i]);
+				a = janus_sdp_attribute_create("rid", "%s recv", stream->rid[index]);
 				m->attributes = g_list_append(m->attributes, a);
 				if(strlen(rids) == 0) {
-					g_strlcat(rids, stream->rid[i], sizeof(rids));
+					g_strlcat(rids, stream->rid[index], sizeof(rids));
 				} else {
 					g_strlcat(rids, ";", sizeof(rids));
-					g_strlcat(rids, stream->rid[i], sizeof(rids));
+					g_strlcat(rids, stream->rid[index], sizeof(rids));
 				}
 			}
 			if(stream->legacy_rid) {
 				a = janus_sdp_attribute_create("simulcast", " recv rid=%s", rids);
 			} else {
-				a = janus_sdp_attribute_create("simulcast", " recv %s", rids);
+				a = janus_sdp_attribute_create("simulcast", "recv %s", rids);
 			}
 			m->attributes = g_list_append(m->attributes, a);
 		}
