@@ -54,6 +54,7 @@ int janus_nanomsg_send_message(janus_transport_session *transport, void *request
 void janus_nanomsg_session_created(janus_transport_session *transport, guint64 session_id);
 void janus_nanomsg_session_over(janus_transport_session *transport, guint64 session_id, gboolean timeout, gboolean claimed);
 void janus_nanomsg_session_claimed(janus_transport_session *transport, guint64 session_id);
+json_t *janus_nanomsg_query_transport(json_t *request);
 
 
 /* Transport setup */
@@ -77,6 +78,8 @@ static janus_transport janus_nanomsg_transport =
 		.session_created = janus_nanomsg_session_created,
 		.session_over = janus_nanomsg_session_over,
 		.session_claimed = janus_nanomsg_session_claimed,
+
+		.query_transport = janus_nanomsg_query_transport,
 	);
 
 /* Transport creator */
@@ -96,6 +99,21 @@ static size_t json_format = JSON_INDENT(3) | JSON_PRESERVE_ORDER;
 
 #define BUFFER_SIZE		8192
 
+/* Parameter validation (for tweaking and queries via Admin API) */
+static struct janus_json_parameter request_parameters[] = {
+	{"request", JSON_STRING, JANUS_JSON_PARAM_REQUIRED}
+};
+static struct janus_json_parameter configure_parameters[] = {
+	{"events", JANUS_JSON_BOOL, 0},
+	{"json", JSON_STRING, 0},
+};
+/* Error codes (for the tweaking and queries via Admin API) */
+#define JANUS_NANOMSG_ERROR_INVALID_REQUEST		411
+#define JANUS_NANOMSG_ERROR_MISSING_ELEMENT		412
+#define JANUS_NANOMSG_ERROR_INVALID_ELEMENT		413
+#define JANUS_NANOMSG_ERROR_UNKNOWN_ERROR		499
+
+
 /* Nanomsg server thread */
 static GThread *nanomsg_thread = NULL;
 void *janus_nanomsg_thread(void *data);
@@ -144,6 +162,7 @@ int janus_nanomsg_init(janus_transport_callbacks *callback, const char *config_p
 		/* Handle configuration */
 		janus_config_print(config);
 		janus_config_category *config_general = janus_config_get_create(config, NULL, janus_config_type_category, "general");
+		janus_config_category *config_admin = janus_config_get_create(config, NULL, janus_config_type_category, "admin");
 
 		janus_config_item *item = janus_config_get(config, config_general, janus_config_type_item, "json");
 		if(item && item->value) {
@@ -225,13 +244,13 @@ int janus_nanomsg_init(janus_transport_callbacks *callback, const char *config_p
 			}
 		}
 		/* Do the same for the Admin API, if enabled */
-		item = janus_config_get(config, config_general, janus_config_type_item, "admin_enabled");
+		item = janus_config_get(config, config_admin, janus_config_type_item, "admin_enabled");
 		if(!item || !item->value || !janus_is_true(item->value)) {
 			JANUS_LOG(LOG_WARN, "Nanomsg server disabled (Admin API)\n");
 		} else {
-			item = janus_config_get(config, config_general, janus_config_type_item, "admin_address");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_address");
 			const char *address = item && item->value ? item->value : NULL;
-			item = janus_config_get(config, config_general, janus_config_type_item, "admin_mode");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_mode");
 			const char *mode = item && item->value ? item->value : NULL;
 			if(mode == NULL)
 				mode = "bind";
@@ -403,6 +422,83 @@ void janus_nanomsg_session_claimed(janus_transport_session *transport, guint64 s
 	/* FIXME Is the above statement accurate? Should we care? Unlike the HTTP transport, there is no hashtable to update */
 }
 
+json_t *janus_nanomsg_query_transport(json_t *request) {
+	if(g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized)) {
+		return NULL;
+	}
+	/* We can use this request to dynamically change the behaviour of
+	 * the transport plugin, and/or query for some specific information */
+	json_t *response = json_object();
+	int error_code = 0;
+	char error_cause[512];
+	JANUS_VALIDATE_JSON_OBJECT(request, request_parameters,
+		error_code, error_cause, TRUE,
+		JANUS_NANOMSG_ERROR_MISSING_ELEMENT, JANUS_NANOMSG_ERROR_INVALID_ELEMENT);
+	if(error_code != 0)
+		goto plugin_response;
+	/* Get the request */
+	const char *request_text = json_string_value(json_object_get(request, "request"));
+	if(!strcasecmp(request_text, "configure")) {
+		/* We only allow for the configuration of some basic properties:
+		 * changing more complex things (e.g., port to bind to, etc.)
+		 * would likely require restarting backends, so just too much */
+		JANUS_VALIDATE_JSON_OBJECT(request, configure_parameters,
+			error_code, error_cause, TRUE,
+			JANUS_NANOMSG_ERROR_MISSING_ELEMENT, JANUS_NANOMSG_ERROR_INVALID_ELEMENT);
+		/* Check if we now need to send events to handlers */
+		json_object_set_new(response, "result", json_integer(200));
+		json_t *notes = NULL;
+		gboolean events = json_is_true(json_object_get(request, "events"));
+		if(events && !gateway->events_is_enabled()) {
+			/* Notify that this will be ignored */
+			notes = json_array();
+			json_array_append_new(notes, json_string("Event handlers disabled at the core level"));
+			json_object_set_new(response, "notes", notes);
+		}
+		if(events != notify_events) {
+			notify_events = events;
+			if(!notify_events && gateway->events_is_enabled()) {
+				JANUS_LOG(LOG_WARN, "Notification of events to handlers disabled for %s\n", JANUS_NANOMSG_NAME);
+			}
+		}
+		const char *indentation = json_string_value(json_object_get(request, "json"));
+		if(indentation != NULL) {
+			if(!strcasecmp(indentation, "indented")) {
+				/* Default: indented, we use three spaces for that */
+				json_format = JSON_INDENT(3) | JSON_PRESERVE_ORDER;
+			} else if(!strcasecmp(indentation, "plain")) {
+				/* Not indented and no new lines, but still readable */
+				json_format = JSON_INDENT(0) | JSON_PRESERVE_ORDER;
+			} else if(!strcasecmp(indentation, "compact")) {
+				/* Compact, so no spaces between separators */
+				json_format = JSON_COMPACT | JSON_PRESERVE_ORDER;
+			} else {
+				JANUS_LOG(LOG_WARN, "Unsupported JSON format option '%s', ignoring tweak\n", indentation);
+				/* Notify that this will be ignored */
+				if(notes == NULL) {
+					notes = json_array();
+					json_object_set_new(response, "notes", notes);
+				}
+				json_array_append_new(notes, json_string("Ignored unsupported indentation format"));
+			}
+		}
+	} else {
+		JANUS_LOG(LOG_VERB, "Unknown request '%s'\n", request_text);
+		error_code = JANUS_NANOMSG_ERROR_INVALID_REQUEST;
+		g_snprintf(error_cause, 512, "Unknown request '%s'", request_text);
+	}
+
+plugin_response:
+		{
+			if(error_code != 0) {
+				/* Prepare JSON error event */
+				json_object_set_new(response, "error_code", json_integer(error_code));
+				json_object_set_new(response, "error", json_string(error_cause));
+			}
+			return response;
+		}
+}
+
 
 /* Thread */
 void *janus_nanomsg_thread(void *data) {
